@@ -53,7 +53,6 @@ class TempDir(object):
 
 
 class TestArgcomplete(unittest.TestCase):
-
     def setUp(self):
         os.environ['_ARGCOMPLETE'] = "yes"
         os.environ['_ARC_DEBUG'] = "yes"
@@ -62,7 +61,7 @@ class TestArgcomplete(unittest.TestCase):
     def tearDown(self):
         pass
 
-    def run_completer(self, parser, command, point=None):
+    def run_completer(self, parser, command, point=None, **kwargs):
         if python2:
             command = unicode(command)
         if point is None:
@@ -77,7 +76,7 @@ class TestArgcomplete(unittest.TestCase):
             os.environ['COMP_POINT'] = point
             os.environ['_ARGCOMPLETE_COMP_WORDBREAKS'] = '"\'@><=;|&(:'
             self.assertRaises(SystemExit, autocomplete, parser, output_stream=t,
-                              exit_method=sys.exit)
+                              exit_method=sys.exit, **kwargs)
             t.seek(0)
             return t.read().decode(locale.getpreferredencoding()).split(IFS)
 
@@ -96,10 +95,10 @@ class TestArgcomplete(unittest.TestCase):
             return parser
 
         expected_outputs = (("prog ", ['--ship', '-h', '--help']),
-            ("prog --shi", ['--ship']),
+            ("prog --shi", ['--ship ']),
             ("prog --ship ", ['submarine', 'speedboat']),
             ("prog --ship s", ['submarine', 'speedboat']),
-            ("prog --ship su", ['submarine']),
+            ("prog --ship su", ['submarine ']),
             )
 
         for cmd, output in expected_outputs:
@@ -113,9 +112,9 @@ class TestArgcomplete(unittest.TestCase):
             return parser
 
         expected_outputs = (("prog ", ['bus', 'car', '-h', '--help']),
-            ("prog bu", ['bus']),
+            ("prog bu", ['bus ']),
             ("prog bus ", ['apple', 'orange', '-h', '--help']),
-            ("prog bus appl", ['apple']),
+            ("prog bus appl", ['apple ']),
             ("prog bus apple ", ['-h', '--help']),
             )
 
@@ -132,9 +131,9 @@ class TestArgcomplete(unittest.TestCase):
             return parser
 
         expected_outputs = (("prog ", ['build', '-h', '--help']),
-            ("prog bu", ['build']),
+            ("prog bu", ['build ']),
             ("prog build ", ['bus', 'car', '--profile', '-h', '--help']),
-            ("prog build ca", ['car']),
+            ("prog build ca", ['car ']),
             ("prog build car ", ['--profile', '-h', '--help']),
             )
 
@@ -187,7 +186,7 @@ class TestArgcomplete(unittest.TestCase):
             return parser
 
         expected_outputs = (("prog ", ['--help', 'eggs', '-h', 'spam', '--age']),
-            ("prog --age 1 eggs", ['eggs']),
+            ("prog --age 1 eggs", ['eggs ']),
             ("prog --age 2 eggs ", ['on a train', 'with a goat', 'on a boat', 'in the rain', '--help', '-h']),
             ("prog eggs ", ['on a train', 'with a goat', 'on a boat', 'in the rain', '--help', '-h']),
             ("prog eggs \"on a", ['\"on a train', '\"on a boat']),
@@ -197,6 +196,9 @@ class TestArgcomplete(unittest.TestCase):
 
         for cmd, output in expected_outputs:
             self.assertEqual(set(self.run_completer(make_parser(), cmd)), set(output))
+            self.assertEqual(set(self.run_completer(make_parser(), cmd, exclude=['-h'])), set(output) - set(['-h']))
+            self.assertEqual(set(self.run_completer(make_parser(), cmd, exclude=['-h', '--help'])),
+                             set(output) - set(['-h', '--help']))
 
     @unittest.skipIf(python2 and sys.getdefaultencoding() != locale.getdefaultlocale()[1],
         "Skip for python 2 due to its text encoding deficiencies")
@@ -219,5 +221,46 @@ class TestArgcomplete(unittest.TestCase):
                 output = [o.decode(locale.getpreferredencoding()) for o in output]
             self.assertEqual(set(self.run_completer(make_parser(), cmd)), set(output))
 
+    def test_custom_validator(self):
+        def make_parser():
+            parser = argparse.ArgumentParser()
+            parser.add_argument('var', choices=['bus', 'car'])
+            parser.add_argument('value', choices=['orange', 'apple'])
+            return parser
+
+        expected_outputs = (("prog ", ['-h', '--help']),
+            ("prog bu", ['']),
+            ("prog bus ", ['-h', '--help']),
+            ("prog bus appl", ['']),
+            ("prog bus apple ", ['-h', '--help']),
+            )
+
+        for cmd, output in expected_outputs:
+            self.assertEqual(set(self.run_completer(make_parser(), cmd, validator=lambda x,y: False) ), set(output))
+
+    def test_different_validators(self):
+        def make_parser():
+            parser = argparse.ArgumentParser()
+            parser.add_argument('var', choices=['bus', 'car'])
+            parser.add_argument('value', choices=['orange', 'apple'])
+            return parser
+
+        validators = (
+                lambda x,y: False,
+                lambda x,y: True,
+                lambda x,y: x.startswith(y),
+        )
+
+        expected_outputs = (("prog ", ['-h', '--help'], validators[0]),
+            ("prog ", ['bus', 'car', '-h', '--help'], validators[1]),
+            ("prog bu", ['bus '], validators[1]),
+            ("prog bus ", ['apple', 'orange', '-h', '--help'], validators[1]),
+            ("prog bus appl", ['apple '], validators[2]),
+            ("prog bus cappl", [''], validators[2]),
+            ("prog bus pple ", ['-h', '--help'], validators[2]),
+            )
+
+        for cmd, output, validator in expected_outputs:
+            self.assertEqual(set(self.run_completer(make_parser(), cmd, validator=validator) ), set(output))
 if __name__ == '__main__':
     unittest.main()
